@@ -37,7 +37,7 @@ class DesktopBot(BaseBot):
         super().__init__()
         self.state = State()
         self.maestro = BotMaestroSDK() if MAESTRO_AVAILABLE else None
-
+        self._interval = 0.005 if platform.system() == "Darwin" else 0.0
         # For parity with Java
         self.addImage = self.add_image
         self.getImageFromMap = self.get_image_from_map
@@ -74,6 +74,7 @@ class DesktopBot(BaseBot):
         self.rightClickAt = self.right_click_at
         self.rightClickRelative = self.right_click_relative
         self.moveAndRightClick = self.right_click
+        pyperclip.determine_clipboard()
 
     ##########
     # Display
@@ -145,6 +146,7 @@ class DesktopBot(BaseBot):
 
         results = [None] * len(labels)
         paths = [self._search_image_file(la) for la in labels]
+        paths = [self._image_path_as_image(la) for la in paths]
 
         if threshold:
             # TODO: Figure out how we should do threshold
@@ -250,6 +252,7 @@ class DesktopBot(BaseBot):
         region = (x, y, w, h)
 
         element_path = self._search_image_file(label)
+        element_path = self._image_path_as_image(element_path)
 
         if threshold:
             # TODO: Figure out how we should do threshold
@@ -308,8 +311,8 @@ class DesktopBot(BaseBot):
                 for itm in items:
                     if itm == item:
                         continue
-                    if (itm.left >= x_start and itm.left <= x_end)\
-                            and (itm.top >= y_start and itm.top <= y_end):
+                    if (itm.left >= x_start and itm.left < x_end)\
+                            and (itm.top >= y_start and itm.top < y_end):
                         similars.append(itm)
                         continue
                 return similars
@@ -335,6 +338,7 @@ class DesktopBot(BaseBot):
         region = (x, y, w, h)
 
         element_path = self._search_image_file(label)
+        element_path = self._image_path_as_image(element_path)
 
         if threshold:
             # TODO: Figure out how we should do threshold
@@ -492,7 +496,12 @@ class DesktopBot(BaseBot):
         if not best:
             print('Warning: Ignoring best=False for now. It will be supported in the future.')
 
-        ele = pyautogui.locateOnScreen(self._search_image_file(label), region=region, confidence=matching)
+        element_path = self._search_image_file(label)
+        element_path = self._image_path_as_image(element_path)
+
+        ele = pyautogui.locateOnScreen(element_path, region=region, confidence=matching)
+        if ele is None:
+            return None, None
         if is_retina():
             ele = ele._replace(left=ele.left / 2.0, top=ele.top / 2.0)
         self.state.element = ele
@@ -545,7 +554,6 @@ class DesktopBot(BaseBot):
     # Mouse
     #######
 
-    @only_if_element
     def click_on(self, label):
         """
         Click on the element.
@@ -554,6 +562,8 @@ class DesktopBot(BaseBot):
             label (str): The image identifier
         """
         x, y = self.get_element_coords_centered(label)
+        if None in (x, y):
+            raise ValueError(f'Element not available. Cannot find {label}.')
         os_compat.click(x, y)
 
     def get_last_x(self):
@@ -788,8 +798,6 @@ class DesktopBot(BaseBot):
             interval_between_clicks (int, optional): The interval between clicks in ms. Defaults to 0.
             wait_after (int, optional): Interval to wait after clicking on the element.
         """
-        x = self.state.x() + x
-        y = self.state.y() + y
         self.click_relative(x, y, wait_after=wait_after, clicks=3, interval_between_clicks=interval_between_clicks,
                             button='right')
 
@@ -830,10 +838,7 @@ class DesktopBot(BaseBot):
         """
         if text:
             pyperclip.copy(text)
-            self.sleep(500)
         self.control_v()
-        delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
-        self.sleep(delay)
 
     def copy_to_clipboard(self, text, wait=0):
         """
@@ -992,7 +997,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('alt', 'space', interval=0.25)
+        pyautogui.hotkey('alt', 'space', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1031,7 +1036,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('alt', 'e', interval=0.25)
+        pyautogui.hotkey('alt', 'e', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1043,7 +1048,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('alt', 'r', interval=0.25)
+        pyautogui.hotkey('alt', 'r', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1055,7 +1060,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('alt', 'f', interval=0.25)
+        pyautogui.hotkey('alt', 'f', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1067,7 +1072,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('alt', 'u', interval=0.25)
+        pyautogui.hotkey('alt', 'u', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1079,7 +1084,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('alt', 'f4', interval=0.25)
+        pyautogui.hotkey('alt', 'f4', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1094,7 +1099,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'c', interval=0.25)
+        pyautogui.hotkey(key, 'c', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
         return self.get_clipboard()
@@ -1110,7 +1115,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'v', interval=0.25)
+        pyautogui.hotkey(key, 'v', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1125,7 +1130,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'a', interval=0.25)
+        pyautogui.hotkey(key, 'a', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1140,7 +1145,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'f', interval=0.25)
+        pyautogui.hotkey(key, 'f', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1155,7 +1160,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'p', interval=0.25)
+        pyautogui.hotkey(key, 'p', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1170,7 +1175,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'u', interval=0.25)
+        pyautogui.hotkey(key, 'u', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1185,7 +1190,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'r', interval=0.25)
+        pyautogui.hotkey(key, 'r', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1200,7 +1205,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 't', interval=0.25)
+        pyautogui.hotkey(key, 't', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1215,7 +1220,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'end', interval=0.25)
+        pyautogui.hotkey(key, 'end', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1230,7 +1235,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'home', interval=0.25)
+        pyautogui.hotkey(key, 'home', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1245,7 +1250,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'w', interval=0.25)
+        pyautogui.hotkey(key, 'w', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1260,7 +1265,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'shift', 'p', interval=0.25)
+        pyautogui.hotkey(key, 'shift', 'p', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1275,7 +1280,7 @@ class DesktopBot(BaseBot):
         key = 'ctrl'
         if platform.system() == 'Darwin':
             key = 'command'
-        pyautogui.hotkey(key, 'shift', 'j', interval=0.25)
+        pyautogui.hotkey(key, 'shift', 'j', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
@@ -1287,7 +1292,7 @@ class DesktopBot(BaseBot):
             wait (int, optional): Wait interval (ms) after task
 
         """
-        pyautogui.hotkey('shift', 'tab', interval=0.25)
+        pyautogui.hotkey('shift', 'tab', interval=self._interval)
         delay = max(0, wait or config.DEFAULT_SLEEP_AFTER_ACTION)
         self.sleep(delay)
 
